/*****************************************************************
 * Copyright (c) ParaSoft Corporation 1991.  All rights reserved.*
 * THIS IS UNPUBLISHED PROPRIETARY SOURCE CODE OF ParaSoft       *
 * The copyright notice above does not evidence any              *
 * actual or intended publication of such source code.           *
 *****************************************************************/

#include <stdio.h>
#include <signal.h>
#include <ctype.h>

#include "lex.h"
#include "error.h"
#include "u.h"
#include "typedef.h"

static ms_on = 0;
static ms_size = 0;
static ms_malloc = 0;
static ms_free = 0;
static ms_realloc = 0;

extern char *malloc (), *realloc ();

char *U_malloc (), *U_realloc ();
char *malloc (), *realloc ();

char *U_strlower (str)  /* Converts string to lowercase */
char *str;
{
	char *p = str;

	if (p == (char*) 0)
		return str;

	while (*p != '\0') {
		if (isupper(*p))
			*p = tolower(*p);
		p++;
	}

	return str;
}

char *U_strupper (str)  /* Converts string to uppercase */
char *str;
{
	char *p = str;

	if (p == (char*) 0)
		return str;

	while (*p != '\0') {
		if (islower (*p))
			*p = toupper (*p);
		p++;
	}

	return str;
}

static handler_ILL ()
{
	syserr ("SIGILL");
	U_exit ();
	exit (777);
}
static handler_TRAP ()
{
	syserr ("SIGTRAP");
	U_exit ();
	exit (777);
}
static handler_IOT ()
{
	syserr ("SIGIOT");
	U_exit ();
	exit (777);
}
static handler_EMT ()
{
	syserr ("SIGEMT");
	U_exit ();
	exit (777);
}
static handler_FPE ()
{
	syserr ("SIGFPE");
	U_exit ();
	exit (777);
}
static handler_BUS ()
{
	syserr ("SIGBUS");
	U_exit ();
	exit (777);
}
static handler_SEGV ()
{
	syserr ("SIGSEGV");
	U_exit ();
	exit (777);
}
static handler_SYS ()
{
	syserr ("SIGSYS");
	U_exit ();
	exit (777);
}

U_no_core_dump ()  /* if called "catches" core dump */
{
#ifndef TURBO_C
	signal (SIGILL, handler_ILL);
	signal (SIGTRAP, handler_TRAP);
	signal (SIGIOT, handler_IOT);
	signal (SIGEMT, handler_EMT);
	signal (SIGFPE, handler_FPE);
	signal (SIGBUS, handler_BUS);
	signal (SIGSEGV, handler_SEGV);
	signal (SIGSYS, handler_SYS);
#endif
}

U_on_core_dump (fun, quit)
PtF fun, quit;
{
#ifndef TURBO_C
	signal (SIGQUIT, quit);
	signal (SIGILL, fun);
	signal (SIGTRAP, fun);
	signal (SIGIOT, fun);
	signal (SIGEMT, fun);
	signal (SIGFPE, fun);
	signal (SIGBUS, fun);
	signal (SIGSEGV, fun);
	signal (SIGSYS, fun);
#endif
}

char *U_str_copy (str)  /* allocates + copies */
char *str;
{
	char *new;

	new = (char*) U_malloc (strlen (str) + 1);
	strcpy (new, str);

	return new;
}

char *U_str_alloc (len) /* allocates + 1 */
int len;
{
	return (char*) U_malloc (len + 1);
}

char *U_str_copy_upper (str) /* allocates + copies + converts to upper */
char *str;
{
    char *new = (char*) U_malloc (strlen (str) + 1);
    char *f = str, *t = new;

    if (f == (char*) 0)
        return str;

    while (*f != '\0')
        if (islower (*f))
            *t++ = toupper (*f++);
        else
            *t++ = *f++;

    return new;
}

U_malloc_stat_init ()
{
	ms_size = ms_malloc = ms_free = ms_realloc = 0;
	ms_on = 1;
}

U_malloc_stat ()  /* prints statistics about U_malloc */
{
	if (ms_on == 0)
		warning ("U_malloc_stat: U_malloc_stat_init not called");

	fprintf (stderr, "\nTotal size allocated %d bytes\n", ms_size);
	fprintf (stderr, "Number of calls to U_malloc %d\n", ms_malloc);
	fprintf (stderr, "Number of calls to U_free %d\n", ms_free);
	fprintf (stderr, "Number of calls to U_realloc %d\n\n", ms_realloc);

	ms_on = 0;
}

char *U_malloc (size)  /* interface to malloc, checks for enough  memory + bzero */
int size;
{
	char *mem;

	if (ms_on) {
		ms_size = ms_size + size;
		ms_malloc++;
	}

	if ((mem = malloc (size)) == (char*) 0) {
		sprintf (serr, "U_malloc: unable to allocate more memory, size = %d", size);
		syserr (serr);
	}

	bzero (mem, size);
	return mem;
}

char* U_realloc (ptr, size, oldsize)  /* interface to realloc */
char *ptr;
int size, oldsize;
{
	char *mem;

	if (ms_on)
		ms_realloc++;

	if ((mem = realloc (ptr, size)) == (char*) 0) {
		sprintf (serr, "U_realloc: unable to allocate more memory, size = %d", size);
		syserr (serr);
	}

	bzero (mem+oldsize, size-oldsize);
	return mem;
}

U_free (mem)  /* interface to free, frees only if mem is not a NULL pointer */
char* mem;
{
	if (ms_on)
		ms_free++;

	if (mem != (char*) 0)
		free (mem);
}

U_check_begin (level)
int level;
{
#ifdef CHECK
	malloc_debug (level);
#endif
}

U_check_end ()
{
#ifdef CHECK
	if (malloc_verify () != 1)
		syserr ("check_end: malloc_verify");
#endif
}

U_StRcMp (s1, s2)  /* doesn't care about case */
char *s1, s2;
{
}

U_exit ()  /* note that it is application specific */
{
	odon ();
#ifdef CHECK
	table_show1 ();
#endif
}

U_coredump ()
{
	char *buf = (char*) 0;
	buf = "core";
}

U_null () {}

int U_strlen (str)
char *str;
{
	if (str == (char*) 0)
		return 0;
	else
		return strlen (str);
}

/* adds str to buf, allocates new buf if necessary in increments of inc_len */

char *U_strcat (buf, str, buf_len, inc_len)
char *buf, *str;
int *buf_len, inc_len;
{
	int buf_used = U_strlen (buf);
	int str_len = U_strlen (str);
	int buf_len_old = *buf_len;
	char *buf_new;

	while (buf_used + str_len + 1 > *buf_len)
		*buf_len = *buf_len + inc_len;

	if (buf_len_old < *buf_len) {
		buf_new = (char*) U_malloc (*buf_len + 1);

		if (buf != (char*) 0) {
			strcpy (buf_new, buf);
			U_free (buf);
		}

		buf = buf_new;
	}

	strcat (buf, str);

	return buf;
}

U_strcmp_nocase (s1, s2)
char *s1, *s2;
{
	s1 = U_str_copy_upper (s1);
	s2 = U_str_copy_upper (s2);

	return strcmp (s1, s2);
}

U_strncmp_nocase (s1, s2, len)
char *s1, *s2;
int len;
{
	s1 = U_str_copy_upper (s1);
	s2 = U_str_copy_upper (s2);

	return strncmp (s1, s2, len);
}

long U_nint (val)
double val;
{
	long ival = (int) val;

	if (val - ival > .5)
		return ival + 1;
	else
		return ival;
}
